# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

from karborclient.common import http as base_client
from karborclient.tests.unit import base
from karborclient.tests.unit import fakes
from karborclient.v1 import client


REQUEST_ID = 'req-test-request-id'
PROJECT_ID = 'efc6a88b-9096-4bb6-8634-cda182a6e12a'


class FakeClient(fakes.FakeClient, client.Client):

    def __init__(self, *args, **kwargs):
        kwargs = {
            'token': 'token',
            'auth': 'auth_url',
            'service_type': 'service_type',
            'endpoint_type': 'endpoint_type',
            'region_name': 'region_name',
            'project_id': PROJECT_ID,
        }
        client.Client.__init__(self, 'http://endpoint', **kwargs)
        self.client = FakeHTTPClient(**kwargs)


class FakeHTTPClient(base_client.HTTPClient):

    def __init__(self, **kwargs):
        self.username = 'username'
        self.password = 'password'
        self.auth_url = 'auth_url'
        self.callstack = []
        self.management_url = 'http://10.0.2.15:8776/v1/fake'
        self.osapi_max_limit = 1000
        self.marker = None

    def _cs_request(self, url, method, **kwargs):
        # Check that certain things are called correctly
        if method in ['GET', 'DELETE']:
            assert 'body' not in kwargs
        elif method == 'PUT':
            assert 'body' in kwargs

        # Call the method
        args = urlparse.parse_qsl(urlparse.urlparse(url)[4])
        kwargs.update(args)
        url_split = url.rsplit('?', 1)
        munged_url = url_split[0]
        if len(url_split) > 1:
            parameters = url_split[1]
            if 'marker' in parameters:
                self.marker = int(parameters.rsplit('marker=', 1)[1])
            else:
                self.marker = None
        else:
            self.marker = None
        munged_url = munged_url.strip('/').replace('/', '_').replace('.', '_')
        munged_url = munged_url.replace('-', '_')

        callback = "%s_%s" % (method.lower(), munged_url)

        if not hasattr(self, callback):
            raise AssertionError('Called unknown API method: %s %s, '
                                 'expected fakes method name: %s' %
                                 (method, url, callback))

        # Note the call
        self.callstack.append((method, url, kwargs.get('body')))
        status, headers, body = getattr(self, callback)(**kwargs)
        # add fake request-id header
        headers['x-openstack-request-id'] = REQUEST_ID
        r = base.TestResponse({
            "status_code": status,
            "text": body,
            "headers": headers,
        })
        return r, body



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'plan': {'name': 'fake_name'}})


class PlansTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_plans_with_marker_limit(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.list(marker=1234, limit=2)
        mock_request.assert_called_with(
            'GET',
            '/plans?limit=2&marker=1234'.format(
                project_id=fakes.PROJECT_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_plans_with_sort_key_dir(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.list(sort_key='id', sort_dir='asc')
        mock_request.assert_called_with(
            'GET',
            '/plans?'
            'sort_dir=asc&sort_key=id'.format(
                project_id=fakes.PROJECT_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_plans_with_invalid_sort_key(self, mock_request):
        self.assertRaises(ValueError,
                          cs.plans.list, sort_key='invalid', sort_dir='asc')

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_create_plan(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.create('Plan name', 'provider_id', '', "", '')
        mock_request.assert_called_with(
            'POST',
            '/plans',
            data={
                'plan': {'provider_id': 'provider_id',
                         'name': 'Plan name',
                         'resources': '',
                         'parameters': '',
                         'description': ''}},
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.raw_request')
    def test_delete_plan(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.delete('1')
        mock_request.assert_called_with(
            'DELETE',
            '/plans/1',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_create_update(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.update('1', {'name': 'Test name.'})
        mock_request.assert_called_with(
            'PUT',
            '/plans/1',
            data={'plan': {'name': 'Test name.'}}, headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_plan(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.get('1')
        mock_request.assert_called_with(
            'GET',
            '/plans/1',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_plan_with_headers(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.plans.get('1', session_id='fake_session_id')
        mock_request.assert_called_with(
            'GET',
            '/plans/1',
            headers={'X-Configuration-Session': 'fake_session_id'})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class Service(base.Resource):
    def __repr__(self):
        return "<Service %s>" % self._info


class ServiceManager(base.ManagerWithFind):
    resource_class = Service

    def enable(self, service_id):
        """Enable the service specified by the service ID

        :param service_id: The ID of the service to enable.
        """
        body = {
            'status': 'enabled'
        }
        return self._update('/os-services/%s' % service_id, body, "service")

    def disable(self, service_id):
        """Disable the service specified by the service ID.

        :param service_id: The ID of the service to disable.
        """
        body = {
            'status': 'disabled'
        }
        return self._update('/os-services/%s' % service_id, body, "service")

    def disable_log_reason(self, service_id, reason):
        """Disable the service with a reason.

        :param service_id: The ID of the service to disable.
        :param reason: The reason for disabling a service.
        """
        body = {
            'status': 'disabled',
            'disabled_reason': reason
        }
        return self._update("/os-services/%s" % service_id, body, "service")

    def list(self, host=None, binary=None):
        """Lists all services."""
        search_opts = {
            'host': host,
            'binary': binary
        }
        resource_type = "os-services"
        url = self._build_list_url(resource_type, search_opts=search_opts)
        return self._list(url, 'services')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import protectables as osc_protectables
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import protectables


PROTECTABLE_LIST_INFO = {
    "protectable_type": [
        "OS::Keystone::Project",
        "OS::Cinder::Volume",
        "OS::Glance::Image",
        "OS::Nova::Server"
    ]
}

PROTECTABLE_SHOW_INFO = {
    "name": "OS::Nova::Server",
    "dependent_types": [
        "OS::Cinder::Volume",
        "OS::Glance::Image"
    ]
}

PROTECTABLE_INSTANCE_LIST_INFO = {
    "id": "25336116-f38e-4c22-81ad-e9b7bd71ba51",
    "type": "OS::Cinder::Volume",
    "name": "System volume",
    "extra_info": {
        "availability_zone": "az1"
    }
}

PROTECTABLE_INSTANCE_SHOW_INFO = {
    "id": "cb4ef2ff-10f5-46c9-bce4-cf7a49c65a01",
    "type": "OS::Nova::Server",
    "name": "My VM",
    "dependent_resources": [{
        "id": "99777fdd-8a5b-45ab-ba2c-52420008103f",
        "type": "OS::Glance::Image",
        "name": "cirros-0.3.4-x86_64-uec"}]
}


class TestProtectables(fakes.TestDataProtection):
    def setUp(self):
        super(TestProtectables, self).setUp()
        cm = self.app.client_manager
        self.protectables_mock = cm.data_protection.protectables
        self.protectables_mock.reset_mock()


class TestListProtectables(TestProtectables):
    def setUp(self):
        super(TestListProtectables, self).setUp()
        self.protectables_mock.list.return_value = [protectables.Protectable(
            None, copy.deepcopy(PROTECTABLE_LIST_INFO))]

        # Command to test
        self.cmd = osc_protectables.ListProtectables(self.app, None)

    def test_protectables_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Protectable type'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [(['OS::Keystone::Project',
                           'OS::Cinder::Volume',
                           'OS::Glance::Image',
                           'OS::Nova::Server'],)]
        self.assertEqual(expected_data, list(data))


class TestShowProtectable(TestProtectables):
    def setUp(self):
        super(TestShowProtectable, self).setUp()
        self.protectables_mock.get.return_value = protectables.Protectable(
            None, copy.deepcopy(PROTECTABLE_SHOW_INFO))
        # Command to test
        self.cmd = osc_protectables.ShowProtectable(self.app, None)

    def test_protectable_show(self):
        arglist = ['OS::Nova::Server']
        verifylist = [('protectable_type', 'OS::Nova::Server')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.protectables_mock.get.assert_called_once_with(
            'OS::Nova::Server')


class TestListProtectableInstances(TestProtectables):
    def setUp(self):
        super(TestListProtectableInstances, self).setUp()
        pm = self.protectables_mock
        pm.list_instances.return_value = [protectables.Instances(
            None, copy.deepcopy(PROTECTABLE_INSTANCE_LIST_INFO)), ]
        # Command to test
        self.cmd = osc_protectables.ListProtectableInstances(self.app, None)

    def test_protectable_instances_list(self):
        arglist = ['OS::Cinder::Volume']
        verifylist = [('protectable_type', 'OS::Cinder::Volume')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.protectables_mock.list_instances.assert_called_once_with(
            'OS::Cinder::Volume', limit=None, marker=None,
            search_opts={'type': None, 'parameters': None},
            sort=None)


class TestShowProtectableInstance(TestProtectables):
    def setUp(self):
        super(TestShowProtectableInstance, self).setUp()
        pm = self.protectables_mock
        pm.get_instance.return_value = protectables.Instances(
            None, copy.deepcopy(PROTECTABLE_INSTANCE_SHOW_INFO))
        # Command to test
        self.cmd = osc_protectables.ShowProtectableInstance(self.app, None)

    def test_protectable_instance_show(self):
        arglist = ['OS::Nova::Server', 'cb4ef2ff-10f5-46c9-bce4-cf7a49c65a01']
        verifylist = [('protectable_type', 'OS::Nova::Server'),
                      ('protectable_id',
                       'cb4ef2ff-10f5-46c9-bce4-cf7a49c65a01')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.protectables_mock.get_instance.assert_called_once_with(
            'OS::Nova::Server', 'cb4ef2ff-10f5-46c9-bce4-cf7a49c65a01',
            search_opts={'parameters': None})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 provider action implementations"""

import functools
import json
from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.i18n import _


class ListProviders(command.Lister):
    _description = _("List providers.")

    log = logging.getLogger(__name__ + ".ListProviders")

    def get_parser(self, prog_name):
        parser = super(ListProviders, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Filters results by a name. Default=None.'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Filters results by a description. Default=None.'),
        )
        parser.add_argument(
            '--marker',
            metavar='<provider>',
            help=_('The last provider ID of the previous page'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-providers>',
            help=_('Maximum number of providers to display'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc) "
                   "(default: name:asc), multiple keys and directions can be "
                   "specified separated by comma"),
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection

        search_opts = {
            'name': parsed_args.name,
            'description': parsed_args.description,
        }

        data = data_protection_client.providers.list(
            search_opts=search_opts, marker=parsed_args.marker,
            limit=parsed_args.limit, sort=parsed_args.sort)

        column_headers = ['Id', 'Name', 'Description']

        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers
                ) for s in data))


class ShowProvider(command.ShowOne):
    _description = "Shows provider details"

    def get_parser(self, prog_name):
        parser = super(ShowProvider, self).get_parser(prog_name)
        parser.add_argument(
            'provider',
            metavar="<provider>",
            help=_('The UUID of the provider.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        provider = osc_utils.find_resource(client.providers,
                                           parsed_args.provider)
        json_dumps = functools.partial(json.dumps, indent=2, sort_keys=True)
        provider._info.pop("links", None)
        if 'extended_info_schema' in provider._info:
            provider._info['extended_info_schema'] = json_dumps(
                provider._info['extended_info_schema'])
        return zip(*sorted(provider._info.items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import abc
import copy

import six
from six.moves.urllib import parse

from karborclient.common.apiclient import exceptions
from karborclient.common import http


SORT_DIR_VALUES = ('asc', 'desc')
SORT_KEY_VALUES = ('id', 'status', 'name', 'created_at')
SORT_KEY_MAPPINGS = {}


def getid(obj):
    """Abstracts the common pattern of allowing both an object or

    an object's ID (UUID) as a parameter when dealing with relationships.
    """

    try:
        return obj.id
    except AttributeError:
        return obj


class Manager(object):
    """Managers interact with a particular type of API (servers, flavors,

    images, etc.) and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api
        if isinstance(self.api, http.SessionClient):
            self.project_id = self.api.get_project_id()
        else:
            self.project_id = self.api.project_id

    def _list(self, url, response_key=None, obj_class=None,
              data=None, headers=None, return_raw=False,):

        if headers is None:
            headers = {}
        resp, body = self.api.json_request('GET', url, headers=headers)

        if obj_class is None:
            obj_class = self.resource_class

        if response_key:
            if response_key not in body:
                body[response_key] = []
            data = body[response_key]
        else:
            data = body
        if return_raw:
            return data
        return [obj_class(self, res, loaded=True) for res in data if res]

    def _delete(self, url, headers=None):
        if headers is None:
            headers = {}
        self.api.raw_request('DELETE', url, headers=headers)

    def _update(self, url, data, response_key=None, headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.json_request('PUT', url, data=data,
                                           headers=headers)
        # PUT requests may not return a body
        if body:
            if response_key:
                return self.resource_class(self, body[response_key])
            return self.resource_class(self, body)

    def _create(self, url, data=None, response_key=None,
                return_raw=False, headers=None):
        if headers is None:
            headers = {}
        if data:
            resp, body = self.api.json_request('POST', url,
                                               data=data, headers=headers)
        else:
            resp, body = self.api.json_request('POST', url, headers=headers)
        if return_raw:
            if response_key:
                return body[response_key]
            return body
        if response_key:
            return self.resource_class(self, body[response_key])
        return self.resource_class(self, body)

    def _get(self, url, response_key=None, return_raw=False, headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.json_request('GET', url, headers=headers)
        if return_raw:
            if response_key:
                return body[response_key]
            return body
        if response_key:
            return self.resource_class(self, body[response_key])
        return self.resource_class(self, body)

    def _build_list_url(self, resource_type, detailed=False,
                        search_opts=None, marker=None, limit=None,
                        sort_key=None, sort_dir=None, sort=None):

        if search_opts is None:
            search_opts = {}

        query_params = {}
        for key, val in search_opts.items():
            if val:
                query_params[key] = val

        if marker:
            query_params['marker'] = marker

        if limit:
            query_params['limit'] = limit

        if sort:
            query_params['sort'] = self._format_sort_param(sort)
        else:
            # sort_key and sort_dir deprecated in kilo, prefer sort
            if sort_key:
                query_params['sort_key'] = self._format_sort_key_param(
                    sort_key)

            if sort_dir:
                query_params['sort_dir'] = self._format_sort_dir_param(
                    sort_dir)

        # Transform the dict to a sequence of two-element tuples in fixed
        # order, then the encoded string will be consistent in Python 2&3.
        query_string = ""
        if query_params:
            params = sorted(query_params.items(), key=lambda x: x[0])
            query_string = "?%s" % parse.urlencode(params)

        detail = ""
        if detailed:
            detail = "/detail"

        return ("/%(resource_type)s%(detail)s"
                "%(query_string)s" %
                {"resource_type": resource_type, "detail": detail,
                 "query_string": query_string})

    def _format_sort_param(self, sort):
        '''Formats the sort information into the sort query string parameter.

        The input sort information can be any of the following:
        - Comma-separated string in the form of <key[:dir]>
        - List of strings in the form of <key[:dir]>
        - List of either string keys, or tuples of (key, dir)

        For example, the following import sort values are valid:
        - 'key1:dir1,key2,key3:dir3'
        - ['key1:dir1', 'key2', 'key3:dir3']
        - [('key1', 'dir1'), 'key2', ('key3', dir3')]

        :param sort: Input sort information
        :returns: Formatted query string parameter or None
        :raise ValueError: If an invalid sort direction or invalid sort key is
                           given
        '''
        if not sort:
            return None

        if isinstance(sort, six.string_types):
            # Convert the string into a list for consistent validation
            sort = [s for s in sort.split(',') if s]

        sort_array = []
        for sort_item in sort:
            if isinstance(sort_item, tuple):
                sort_key = sort_item[0]
                sort_dir = sort_item[1]
            else:
                sort_key, _sep, sort_dir = sort_item.partition(':')
            sort_key = sort_key.strip()
            if sort_key in SORT_KEY_VALUES:
                sort_key = SORT_KEY_MAPPINGS.get(sort_key, sort_key)
            else:
                raise ValueError('sort_key must be one of the following: %s.'
                                 % ', '.join(SORT_KEY_VALUES))
            if sort_dir:
                sort_dir = sort_dir.strip()
                if sort_dir not in SORT_DIR_VALUES:
                    msg = ('sort_dir must be one of the following: %s.'
                           % ', '.join(SORT_DIR_VALUES))
                    raise ValueError(msg)
                sort_array.append('%s:%s' % (sort_key, sort_dir))
            else:
                sort_array.append(sort_key)
        return ','.join(sort_array)

    def _format_sort_key_param(self, sort_key):
        if sort_key in SORT_KEY_VALUES:
            return SORT_KEY_MAPPINGS.get(sort_key, sort_key)

        msg = ('sort_key must be one of the following: %s.' %
               ', '.join(SORT_KEY_VALUES))
        raise ValueError(msg)

    def _format_sort_dir_param(self, sort_dir):
        if sort_dir in SORT_DIR_VALUES:
            return sort_dir

        msg = ('sort_dir must be one of the following: %s.'
               % ', '.join(SORT_DIR_VALUES))
        raise ValueError(msg)


@six.add_metaclass(abc.ABCMeta)
class ManagerWithFind(Manager):
    """Manager with additional `find()`/`findall()` methods."""

    @abc.abstractmethod
    def list(self):
        pass

    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        rl = self.findall(**kwargs)
        num = len(rl)

        if num == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(msg)
        elif num > 1:
            raise exceptions.NoUniqueMatch
        else:
            return self.get(rl[0].id)

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        for obj in self.list():
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue

        return found


class Resource(object):
    """A resource represents a particular instance of an object (tenant, user,

    etc). This is pretty much just a bag for attributes.

    :param manager: Manager object
    :param info: dictionary representing resource attributes
    :param loaded: prevent lazy-loading if set to True
    """
    def __init__(self, manager, info, loaded=False):
        self.manager = manager
        self._info = info
        self._add_details(info)
        self._loaded = loaded

    def _add_details(self, info):
        for k, v in info.items():
            setattr(self, k, v)

    def __setstate__(self, d):
        for k, v in d.items():
            setattr(self, k, v)

    def __getattr__(self, k):
        if k not in self.__dict__:
            # NOTE(bcwaldon): disallow lazy-loading if already loaded once
            if not self.is_loaded():
                self.get()
                return self.__getattr__(k)
            raise AttributeError(k)
        else:
            return self.__dict__[k]

    def __repr__(self):
        reprkeys = sorted(k for k in self.__dict__.keys() if k[0] != '_' and
                          k != 'manager')
        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
        return "<%s %s>" % (self.__class__.__name__, info)

    def get(self):
        # set_loaded() first ... so if we have to bail, we know we tried.
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.id)
        if new:
            self._add_details(new._info)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        return self._info == other._info

    def __ne__(self, other):
        return not self.__eq__(other)

    def is_loaded(self):
        return self._loaded

    def set_loaded(self, val):
        self._loaded = val

    def to_dict(self):
        return copy.deepcopy(self._info)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import http
from karborclient.v1 import checkpoints
from karborclient.v1 import operation_logs
from karborclient.v1 import plans
from karborclient.v1 import protectables
from karborclient.v1 import providers
from karborclient.v1 import quota_classes
from karborclient.v1 import quotas
from karborclient.v1 import restores
from karborclient.v1 import scheduled_operations
from karborclient.v1 import services
from karborclient.v1 import triggers
from karborclient.v1 import verifications


class Client(object):
    """Client for the karbor v1 API.

    :param string endpoint: A user-supplied endpoint URL for the service.
    :param string token: Token for authentication.
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    """

    def __init__(self, *args, **kwargs):
        """Initialize a new client for the karbor v1 API."""
        self.http_client = http._construct_http_client(*args, **kwargs)
        self.plans = plans.PlanManager(self.http_client)
        self.restores = restores.RestoreManager(self.http_client)
        self.protectables = protectables.ProtectableManager(self.http_client)
        self.providers = providers.ProviderManager(self.http_client)
        self.checkpoints = checkpoints.CheckpointManager(self.http_client)
        self.triggers = triggers.TriggerManager(self.http_client)
        self.scheduled_operations = \
            scheduled_operations.ScheduledOperationManager(self.http_client)
        self.operation_logs = \
            operation_logs.OperationLogManager(self.http_client)
        self.verifications = verifications.VerificationManager(
            self.http_client)
        self.services = services.ServiceManager(self.http_client)
        self.quotas = quotas.QuotaManager(self.http_client)
        self.quota_classes = quota_classes.QuotaClassManager(self.http_client)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import copy

from karborclient.osc.v1 import services as osc_services
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import services

SERVICE_INFO = {
    "status": "enabled",
    "binary": "karbor-operationengine",
    "state": "up",
    "updated_at": "2017-10-25T07:06:58.000000",
    "host": "fake_host",
    "disabled_reason": None,
    "id": 1
}


class TestServices(fakes.TestDataProtection):
    def setUp(self):
        super(TestServices, self).setUp()
        self.services_mock = self.app.client_manager.data_protection.services
        self.services_mock.reset_mock()


class TestListServices(TestServices):
    def setUp(self):
        super(TestListServices, self).setUp()
        self.services_mock.list.return_value = [
            services.Service(None, copy.deepcopy(SERVICE_INFO))]
        self.cmd = osc_services.ListServices(self.app, None)

    def test_services_list(self):
        arg_list = ['--host', 'fake_host']
        verify_list = [('host', 'fake_host')]

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        columns, data = self.cmd.take_action(parsed_args)
        expected_columns = (["Id", "Binary", "Host", "Status", "State",
                             "Updated_at", "Disabled Reason"])
        self.assertEqual(expected_columns, columns)
        expected_data = [(1,
                          "karbor-operationengine",
                          "fake_host",
                          "enabled",
                          "up",
                          "2017-10-25T07:06:58.000000",
                          None
                          )]
        self.assertEqual(expected_data, list(data))


class TestEnableService(TestServices):
    def setUp(self):
        super(TestEnableService, self).setUp()
        self.services_mock.enable.return_value = services.Service(
            None, copy.deepcopy(SERVICE_INFO))
        self.cmd = osc_services.EnableService(self.app, None)

    def test_enable_service(self):
        arg_list = ['1']
        verify_list = [('service_id', '1')]
        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        self.cmd.take_action(parsed_args)
        self.services_mock.enable.assert_called_once_with('1')


class TestDisableService(TestServices):
    def setUp(self):
        super(TestDisableService, self).setUp()
        self.services_mock.disable.return_value = services.Service(
            None, copy.deepcopy(SERVICE_INFO))
        self.services_mock.disable_log_reason.return_value = services.Service(
            None, copy.deepcopy(SERVICE_INFO))
        self.cmd = osc_services.DisableService(self.app, None)

    def test_disable_service(self):
        arg_list = ['1']
        verify_list = [('service_id', '1')]
        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        self.cmd.take_action(parsed_args)
        self.services_mock.disable.assert_called_once_with('1')

    def test_disable_service_with_reason(self):
        arg_list = ['1', '--reason', 'fake_reason']
        verify_list = [('service_id', '1'), ('reason', 'fake_reason')]
        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        self.cmd.take_action(parsed_args)
        self.services_mock.disable_log_reason.assert_called_once_with(
            '1', 'fake_reason')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import plans as osc_plans
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import plans


PLAN_INFO = {
    "status": "suspended",
    "provider_id": "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
    "description": "",
    "parameters": {},
    "id": "204c825e-eb2f-4609-95ab-70b3caa43ac8",
    "resources": [{
        'type': 'OS::Cinder::Volume',
        'id': '71bfe64a-e0b9-4a91-9e15-a7fc9ab31b14',
        'name': 'testsinglevolume'}],
    "name": "OS Volume protection plan."
}


class TestPlans(fakes.TestDataProtection):
    def setUp(self):
        super(TestPlans, self).setUp()
        self.plans_mock = self.app.client_manager.data_protection.plans
        self.plans_mock.reset_mock()


class TestListPlans(TestPlans):
    def setUp(self):
        super(TestListPlans, self).setUp()
        self.plans_mock.list.return_value = [plans.Plan(
            None, copy.deepcopy(PLAN_INFO))]

        # Command to test
        self.cmd = osc_plans.ListPlans(self.app, None)

    def test_plans_list(self):
        arglist = ['--status', 'suspended']
        verifylist = [('status', 'suspended')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Name', 'Description', 'Provider id', 'Status'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [("204c825e-eb2f-4609-95ab-70b3caa43ac8",
                          "OS Volume protection plan.",
                          "",
                          "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
                          "suspended")]
        self.assertEqual(expected_data, list(data))


class TestCreatePlan(TestPlans):
    def setUp(self):
        super(TestCreatePlan, self).setUp()
        self.plans_mock.create.return_value = plans.Plan(
            None, copy.deepcopy(PLAN_INFO))
        # Command to test
        self.cmd = osc_plans.CreatePlan(self.app, None)

    def test_plan_create(self):
        arglist = ['OS Volume protection plan.',
                   'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   "'71bfe64a-e0b9-4a91-9e15-a7fc9ab31b14'="
                   "'OS::Cinder::Volume'='testsinglevolume'"]
        verifylist = [('name', 'OS Volume protection plan.'),
                      ('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('resources', "'71bfe64a-e0b9-4a91-9e15-a7fc9ab31b14'="
                                    "'OS::Cinder::Volume'='testsinglevolume'")]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plans_mock.create.assert_called_once_with(
            'OS Volume protection plan.',
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            [{'id': "'71bfe64a-e0b9-4a91-9e15-a7fc9ab31b14'",
              'type': "'OS::Cinder::Volume'",
              'name': "'testsinglevolume'"}],
            {}, description=None)


class TestUpdatePlan(TestPlans):
    def setUp(self):
        super(TestUpdatePlan, self).setUp()
        self.plans_mock.get.return_value = plans.Plan(
            None, copy.deepcopy(PLAN_INFO))
        self.plans_mock.update.return_value = plans.Plan(
            None, copy.deepcopy(PLAN_INFO))
        # Command to test
        self.cmd = osc_plans.UpdatePlan(self.app, None)

    def test_plan_update(self):
        arglist = ['204c825e-eb2f-4609-95ab-70b3caa43ac8',
                   '--status', 'started']
        verifylist = [('plan_id', '204c825e-eb2f-4609-95ab-70b3caa43ac8'),
                      ('status', 'started')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plans_mock.update.assert_called_once_with(
            '204c825e-eb2f-4609-95ab-70b3caa43ac8',
            {'status': 'started'})


class TestDeletePlan(TestPlans):
    def setUp(self):
        super(TestDeletePlan, self).setUp()
        self.plans_mock.get.return_value = plans.Plan(
            None, copy.deepcopy(PLAN_INFO))
        # Command to test
        self.cmd = osc_plans.DeletePlan(self.app, None)

    def test_plan_delete(self):
        arglist = ['204c825e-eb2f-4609-95ab-70b3caa43ac8']
        verifylist = [('plan', ['204c825e-eb2f-4609-95ab-70b3caa43ac8'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plans_mock.delete.assert_called_once_with(
            '204c825e-eb2f-4609-95ab-70b3caa43ac8')


class TestShowPlan(TestPlans):
    def setUp(self):
        super(TestShowPlan, self).setUp()
        self._plan_info = copy.deepcopy(PLAN_INFO)
        self.plans_mock.get.return_value = plans.Plan(
            None, self._plan_info)

        # Command to test
        self.cmd = osc_plans.ShowPlan(self.app, None)

    def test_plan_show(self):
        arglist = ['204c825e-eb2f-4609-95ab-70b3caa43ac8']
        verifylist = [('plan', '204c825e-eb2f-4609-95ab-70b3caa43ac8')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            'description', 'id', 'name', 'parameters', 'provider_id',
            'resources', 'status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        self.assertEqual(self._plan_info['description'], data[0])
        self.assertEqual(self._plan_info['id'], data[1])
        self.assertEqual(self._plan_info['name'], data[2])
        self.assertEqual(self._plan_info['parameters'], data[3])
        self.assertEqual(self._plan_info['provider_id'], data[4])
        self.assertEqual(self._plan_info['resources'], data[5])
        self.assertEqual(self._plan_info['status'], data[6])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import providers as osc_providers
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import providers


PROVIDER_INFO = {
    "id": "2220f8b1-975d-4621-a872-fa9afb43cb6c",
    "name": "OS Infra Provider",
    "description": "provider description",
    "extended_info_schema": {
        "options_schema": {
            "OS::Cinder::Volume": {
                "required": [
                    "backup_mode"
                ],
                "type": "object",
                "properties": {
                    "backup_mode": {
                        "default": "auto",
                        "enum": [
                            "full",
                            "incremental",
                            "auto"
                        ],
                        "type": "string",
                        "description": "The backup mode.",
                        "title": "Backup Mode"
                    }
                },
                "title": "Cinder Protection Options"
            }
        },
        "saved_info_schema": {
            "OS::Cinder::Volume": {
                "required": [
                    "name"
                ],
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "description": "The name for this backup.",
                        "title": "Name"
                    }
                },
                "title": "Cinder Protection Saved Info"
            }
        },
        "restore_schema": {
            "OS::Cinder::Volume": {
                "type": "object",
                "properties": {
                    "restore_name": {
                        "type": "string",
                        "description": "The name of the restored volume.",
                        "title": "Restore Name"
                    }
                },
                "title": "Cinder Protection Restore"
            }
        }
    }
}


class TestProviders(fakes.TestDataProtection):
    def setUp(self):
        super(TestProviders, self).setUp()
        self.providers_mock = self.app.client_manager.data_protection.providers
        self.providers_mock.reset_mock()


class TestListProviders(TestProviders):
    def setUp(self):
        super(TestListProviders, self).setUp()
        self.providers_mock.list.return_value = [providers.Provider(
            None, copy.deepcopy(PROVIDER_INFO))]

        # Command to test
        self.cmd = osc_providers.ListProviders(self.app, None)

    def test_providers_list(self):
        arglist = ['--name', 'OS Infra Provider']
        verifylist = [('name', 'OS Infra Provider')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Name', 'Description'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [("2220f8b1-975d-4621-a872-fa9afb43cb6c",
                          "OS Infra Provider",
                          "provider description")]
        self.assertEqual(expected_data, list(data))


class TestShowProvider(TestProviders):
    def setUp(self):
        super(TestShowProvider, self).setUp()
        self._provider_info = copy.deepcopy(PROVIDER_INFO)
        self.providers_mock.get.return_value = providers.Provider(
            None, self._provider_info)

        # Command to test
        self.cmd = osc_providers.ShowProvider(self.app, None)

    def test_provider_show(self):
        arglist = ['2220f8b1-975d-4621-a872-fa9afb43cb6c']
        verifylist = [('provider', '2220f8b1-975d-4621-a872-fa9afb43cb6c')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            'description', 'extended_info_schema', 'id', 'name')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        self.assertEqual(self._provider_info['description'], data[0])
        self.assertEqual(self._provider_info['extended_info_schema'], data[1])
        self.assertEqual(self._provider_info['id'], data[2])
        self.assertEqual(self._provider_info['name'], data[3])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket

import mock
import testtools

from karborclient.common.apiclient import exceptions as exc
from karborclient.common import http
from karborclient.tests.unit import fakes


@mock.patch('karborclient.common.http.requests.request')
class HttpClientTest(testtools.TestCase):

    # Patch os.environ to avoid required auth info.
    def setUp(self):
        super(HttpClientTest, self).setUp()

    def test_http_raw_request(self, mock_request):
        headers = {'User-Agent': 'python-karborclient'}
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {},
                '')

        client = http.HTTPClient('http://example.com:8082')
        resp = client.raw_request('GET', '')
        self.assertEqual(200, resp.status_code)
        self.assertEqual('', ''.join([x for x in resp.content]))
        mock_request.assert_called_with('GET', 'http://example.com:8082',
                                        allow_redirects=False,
                                        headers=headers)

    def test_token_or_credentials(self, mock_request):
        # Record a 200
        fake200 = fakes.FakeHTTPResponse(
            200, 'OK',
            {},
            '')

        mock_request.side_effect = [fake200, fake200, fake200]

        # Replay, create client, assert
        client = http.HTTPClient('http://example.com:8082')
        resp = client.raw_request('GET', '')
        self.assertEqual(200, resp.status_code)

        client.username = 'user'
        client.password = 'pass'
        resp = client.raw_request('GET', '')
        self.assertEqual(200, resp.status_code)

        client.auth_token = 'abcd1234'
        resp = client.raw_request('GET', '')
        self.assertEqual(200, resp.status_code)

        # no token or credentials
        mock_request.assert_has_calls([
            mock.call('GET', 'http://example.com:8082',
                      allow_redirects=False,
                      headers={'User-Agent': 'python-karborclient'}),
            mock.call('GET', 'http://example.com:8082',
                      allow_redirects=False,
                      headers={'User-Agent': 'python-karborclient',
                               'X-Auth-Key': 'pass',
                               'X-Auth-User': 'user'}),
            mock.call('GET', 'http://example.com:8082',
                      allow_redirects=False,
                      headers={'User-Agent': 'python-karborclient',
                               'X-Auth-Token': 'abcd1234'})
        ])

    def test_region_name(self, mock_request):
        # Record a 200
        fake200 = fakes.FakeHTTPResponse(
            200, 'OK',
            {},
            '')

        mock_request.return_value = fake200

        client = http.HTTPClient('http://example.com:8082')
        client.region_name = 'RegionOne'
        resp = client.raw_request('GET', '')
        self.assertEqual(200, resp.status_code)

        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'X-Region-Name': 'RegionOne',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request(self, mock_request):
        # Record a 200
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')
        client = http.HTTPClient('http://example.com:8082')
        resp, body = client.json_request('GET', '')
        self.assertEqual(200, resp.status_code)
        self.assertEqual({}, body)

        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request_argument_passed_to_requests(self, mock_request):
        """Check that we have sent the proper arguments to requests."""
        # Record a 200
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')

        client = http.HTTPClient('http://example.com:8082')
        client.verify_cert = True
        client.cert_file = 'RANDOM_CERT_FILE'
        client.key_file = 'RANDOM_KEY_FILE'
        client.auth_url = 'http://AUTH_URL'
        resp, body = client.json_request('GET', '', data='text')
        self.assertEqual(200, resp.status_code)
        self.assertEqual({}, body)

        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            cert=('RANDOM_CERT_FILE', 'RANDOM_KEY_FILE'),
            verify=True,
            data='"text"',
            headers={'Content-Type': 'application/json',
                     'X-Auth-Url': 'http://AUTH_URL',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request_w_req_body(self, mock_request):
        # Record a 200
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')

        client = http.HTTPClient('http://example.com:8082')
        resp, body = client.json_request('GET', '', data='test-body')
        self.assertEqual(200, resp.status_code)
        self.assertEqual({}, body)
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            data='"test-body"',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request_non_json_resp_cont_type(self, mock_request):
        # Record a 200
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'not/json'},
                '{}')

        client = http.HTTPClient('http://example.com:8082')
        resp, body = client.json_request('GET', '', data='test-data')
        self.assertEqual(200, resp.status_code)
        self.assertIsNone(body)
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082', data='"test-data"',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request_invalid_json(self, mock_request):
        # Record a 200
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                'invalid-json')

        client = http.HTTPClient('http://example.com:8082')
        resp, body = client.json_request('GET', '')
        self.assertEqual(200, resp.status_code)
        self.assertEqual('invalid-json', body)
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_manual_redirect_delete(self, mock_request):
        mock_request.side_effect = [
            fakes.FakeHTTPResponse(
                302, 'Found',
                {'location': 'http://example.com:8082/foo/bar'},
                ''),
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')]

        client = http.HTTPClient('http://example.com:8082/foo')
        resp, body = client.json_request('DELETE', '')

        self.assertEqual(200, resp.status_code)
        mock_request.assert_has_calls([
            mock.call('DELETE', 'http://example.com:8082/foo',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'}),
            mock.call('DELETE', 'http://example.com:8082/foo/bar',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'})
        ])

    def test_http_manual_redirect_post(self, mock_request):
        mock_request.side_effect = [
            fakes.FakeHTTPResponse(
                302, 'Found',
                {'location': 'http://example.com:8082/foo/bar'},
                ''),
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')]

        client = http.HTTPClient('http://example.com:8082/foo')
        resp, body = client.json_request('POST', '')

        self.assertEqual(200, resp.status_code)
        mock_request.assert_has_calls([
            mock.call('POST', 'http://example.com:8082/foo',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'}),
            mock.call('POST', 'http://example.com:8082/foo/bar',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'})
        ])

    def test_http_manual_redirect_put(self, mock_request):
        mock_request.side_effect = [
            fakes.FakeHTTPResponse(
                302, 'Found',
                {'location': 'http://example.com:8082/foo/bar'},
                ''),
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')]

        client = http.HTTPClient('http://example.com:8082/foo')
        resp, body = client.json_request('PUT', '')

        self.assertEqual(200, resp.status_code)
        mock_request.assert_has_calls([
            mock.call('PUT', 'http://example.com:8082/foo',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'}),
            mock.call('PUT', 'http://example.com:8082/foo/bar',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'})
        ])

    def test_http_manual_redirect_prohibited(self, mock_request):
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                302, 'Found',
                {'location': 'http://example.com:8082/'},
                '')
        client = http.HTTPClient('http://example.com:8082/foo')
        self.assertRaises(exc.EndpointException,
                          client.json_request, 'DELETE', '')
        mock_request.assert_called_once_with(
            'DELETE', 'http://example.com:8082/foo',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_manual_redirect_error_without_location(self, mock_request):
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                302, 'Found',
                {},
                '')
        client = http.HTTPClient('http://example.com:8082/foo')
        self.assertRaises(exc.EndpointException,
                          client.json_request, 'DELETE', '')
        mock_request.assert_called_once_with(
            'DELETE', 'http://example.com:8082/foo',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_json_request_redirect(self, mock_request):
        # Record the 302
        mock_request.side_effect = [
            fakes.FakeHTTPResponse(
                302, 'Found',
                {'location': 'http://example.com:8082'},
                ''),
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')]

        client = http.HTTPClient('http://example.com:8082')
        resp, body = client.json_request('GET', '')
        self.assertEqual(200, resp.status_code)
        self.assertEqual({}, body)

        mock_request.assert_has_calls([
            mock.call('GET', 'http://example.com:8082',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'}),
            mock.call('GET', 'http://example.com:8082',
                      allow_redirects=False,
                      headers={'Content-Type': 'application/json',
                               'User-Agent': 'python-karborclient'})
        ])

    def test_http_404_json_request(self, mock_request):
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                404, 'Not Found', {'content-type': 'application/json'},
                '{}')

        client = http.HTTPClient('http://example.com:8082')
        e = self.assertRaises(exc.HTTPClientError,
                              client.json_request, 'GET', '')
        # Assert that the raised exception can be converted to string
        self.assertIsNotNone(str(e))
        # Record a 404
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_http_300_json_request(self, mock_request):
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                300, 'OK', {'content-type': 'application/json'},
                '{}')
        client = http.HTTPClient('http://example.com:8082')
        e = self.assertRaises(
            exc.MultipleChoices, client.json_request, 'GET', '')
        # Assert that the raised exception can be converted to string
        self.assertIsNotNone(str(e))

        # Record a 300
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'})

    def test_fake_json_request(self, mock_request):
        headers = {'User-Agent': 'python-karborclient'}
        mock_request.side_effect = [socket.gaierror]

        client = http.HTTPClient('fake://example.com:8082')
        self.assertRaises(exc.EndpointException,
                          client._http_request, "/", "GET")
        mock_request.assert_called_once_with('GET', 'fake://example.com:8082/',
                                             allow_redirects=False,
                                             headers=headers)

    def test_http_request_socket_error(self, mock_request):
        headers = {'User-Agent': 'python-karborclient'}
        mock_request.side_effect = [socket.gaierror]

        client = http.HTTPClient('http://example.com:8082')
        self.assertRaises(exc.EndpointException,
                          client._http_request, "/", "GET")
        mock_request.assert_called_once_with('GET', 'http://example.com:8082/',
                                             allow_redirects=False,
                                             headers=headers)

    def test_http_request_socket_timeout(self, mock_request):
        headers = {'User-Agent': 'python-karborclient'}
        mock_request.side_effect = [socket.timeout]

        client = http.HTTPClient('http://example.com:8082')
        self.assertRaises(exc.ConnectionRefused,
                          client._http_request, "/", "GET")
        mock_request.assert_called_once_with('GET', 'http://example.com:8082/',
                                             allow_redirects=False,
                                             headers=headers)

    def test_http_request_specify_timeout(self, mock_request):
        mock_request.return_value = \
            fakes.FakeHTTPResponse(
                200, 'OK',
                {'content-type': 'application/json'},
                '{}')

        client = http.HTTPClient('http://example.com:8082', timeout='123')
        resp, body = client.json_request('GET', '')
        self.assertEqual(200, resp.status_code)
        self.assertEqual({}, body)
        mock_request.assert_called_once_with(
            'GET', 'http://example.com:8082',
            allow_redirects=False,
            headers={'Content-Type': 'application/json',
                     'User-Agent': 'python-karborclient'},
            timeout=float(123))

    def test_get_system_ca_file(self, mock_request):
        chosen = '/etc/ssl/certs/ca-certificates.crt'
        with mock.patch('os.path.exists') as mock_os:
            mock_os.return_value = chosen

            ca = http.get_system_ca_file()
            self.assertEqual(chosen, ca)

            mock_os.assert_called_once_with(chosen)

    def test_insecure_verify_cert_None(self, mock_request):
        client = http.HTTPClient('https://foo', insecure=True)
        self.assertFalse(client.verify_cert)

    def test_passed_cert_to_verify_cert(self, mock_request):
        client = http.HTTPClient('https://foo', cacert="NOWHERE")
        self.assertEqual("NOWHERE", client.verify_cert)

        with mock.patch('karborclient.common.http.get_system_ca_file') as gsf:
            gsf.return_value = "SOMEWHERE"
            client = http.HTTPClient('https://foo')
            self.assertEqual("SOMEWHERE", client.verify_cert)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils


class FakeHTTPResponse(object):

    version = 1.1

    def __init__(self, status_code, reason, headers, content):
        self.headers = headers
        self.content = content
        self.status_code = status_code
        self.reason = reason
        self.raw = FakeRaw()

    def getheader(self, name, default=None):
        return self.headers.get(name, default)

    def getheaders(self):
        return self.headers.items()

    def read(self, amt=None):
        b = self.content
        self.content = None
        return b

    def iter_content(self, chunksize):
        return self.content

    def json(self):
        return jsonutils.loads(self.content)


class FakeRaw(object):
    version = 110


class FakeClient(object):

    def _dict_match(self, partial, real):

        result = True
        try:
            for key, value in partial.items():
                if type(value) is dict:
                    result = self._dict_match(value, real[key])
                else:
                    assert real[key] == value
                    result = True
        except (AssertionError, KeyError):
            result = False
        return result

    def assert_called(self, method, url, body=None,
                      partial_body=None, pos=-1, **kwargs):
        """Assert than an API method was just called.

        """

        expected = (method, url)
        called = self.client.callstack[pos][0:2]

        assert self.client.callstack, ("Expected %s %s but no calls "
                                       "were made." % expected)

        assert expected == called, 'Expected %s %s; got %s %s' % (
            expected + called)

        if body is not None:
            assert self.client.callstack[pos][2] == body

        if partial_body is not None:
            try:
                assert self._dict_match(partial_body,
                                        self.client.callstack[pos][2])
            except AssertionError:
                print(self.client.callstack[pos][2])
                print("does not contain")
                print(partial_body)
                raise

    def assert_called_anytime(self, method, url, body=None, partial_body=None):
        """Assert than an API method was called anytime in the test.

        """

        expected = (method, url)

        assert self.client.callstack, ("Expected %s %s but no calls "
                                       "were made." % expected)

        found = False
        for entry in self.client.callstack:
            if expected == entry[0:2]:
                found = True
                break

        assert found, 'Expected %s %s; got %s' % (
            expected + (self.client.callstack, ))

        if body is not None:
            try:
                assert entry[2] == body
            except AssertionError:
                print(entry[2])
                print("!=")
                print(body)
                raise

        if partial_body is not None:
            try:
                assert self._dict_match(partial_body, entry[2])
            except AssertionError:
                print(entry[2])
                print("does not contain")
                print(partial_body)
                raise

    def clear_callstack(self):
        self.client.callstack = []

    def authenticate(self):
        pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from karborclient.common import base


class Protectable(base.Resource):
    def __repr__(self):
        return "<Protectable %s>" % self._info


class Instances(base.Resource):
    def __repr__(self):
        return "<Instances %s>" % self._info


class ProtectableManager(base.ManagerWithFind):
    resource_class = Protectable

    def get(self, protectable_type, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/protectables/{protectable_type}".format(
            protectable_type=protectable_type)
        return self._get(url, response_key="protectable_type", headers=headers)

    def list(self):
        url = "/protectables"
        protectables = self._list(url, 'protectable_type', return_raw=True)

        protectables_list = []
        for protectable in protectables:
            protectable_dict = {}
            protectable_dict['protectable_type'] = protectable
            protectables_list.append(Protectable(self, protectable_dict))
        return protectables_list

    def list_instances(self, protectable_type, search_opts=None, marker=None,
                       limit=None, sort_key=None, sort_dir=None, sort=None):
        """Lists all instances.

        :param protectable_type:
        :param search_opts: Search options to filter out instances.
        :param marker: Begin returning volumes that appear later in the
                       instances list.
        :param limit: Maximum number of instances to return.
        :param sort_key: Key to be sorted; deprecated in kilo
        :param sort_dir: Sort direction, should be 'desc' or 'asc'; deprecated
                         in kilo
        :param sort: Sort information
        :rtype: list of :class:`Instances`
        """

        url = self._build_instances_list_url(
            protectable_type,
            search_opts=search_opts, marker=marker,
            limit=limit, sort_key=sort_key,
            sort_dir=sort_dir, sort=sort)
        return self._list(url, response_key='instances', obj_class=Instances)

    def get_instance(self, type, id, search_opts=None, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}

        if search_opts is None:
            search_opts = {}
        query_params = {}
        for key, val in search_opts.items():
            if val:
                query_params[key] = val
        query_string = ""
        if query_params:
            params = sorted(query_params.items(), key=lambda x: x[0])
            query_string = "?%s" % parse.urlencode(params)

        url = ("/protectables/{protectable_type}/instances/"
               "{protectable_id}{query_string}").format(
            protectable_type=type, protectable_id=id,
            query_string=query_string)
        return self._get(url, response_key="instance", headers=headers)

    def _build_instances_list_url(self, protectable_type,
                                  search_opts=None, marker=None, limit=None,
                                  sort_key=None, sort_dir=None, sort=None):

        if search_opts is None:
            search_opts = {}

        query_params = {}
        for key, val in search_opts.items():
            if val:
                query_params[key] = val

        if marker:
            query_params['marker'] = marker

        if limit:
            query_params['limit'] = limit

        if sort:
            query_params['sort'] = self._format_sort_param(sort)
        else:
            # sort_key and sort_dir deprecated in kilo, prefer sort
            if sort_key:
                query_params['sort_key'] = self._format_sort_key_param(
                    sort_key)

            if sort_dir:
                query_params['sort_dir'] = self._format_sort_dir_param(
                    sort_dir)

        # Transform the dict to a sequence of two-element tuples in fixed
        # order, then the encoded string will be consistent in Python 2&3.
        query_string = ""
        if query_params:
            params = sorted(query_params.items(), key=lambda x: x[0])
            query_string = "?%s" % parse.urlencode(params)

        return ("/protectables/%(protectable_type)s"
                "/instances%(query_string)s" %
                {"protectable_type": protectable_type,
                 "query_string": query_string})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class Quota(base.Resource):
    def __repr__(self):
        return "<Quota %s>" % self._info


class QuotaManager(base.ManagerWithFind):
    resource_class = Quota

    def list(self):
        pass

    def update(self, project_id, data):

        body = {"quota": data}

        return self._update('/quotas/{project_id}'
                            .format(project_id=project_id),
                            body, "quota")

    def get(self, project_id, detail, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        base_url = "/quotas/{project_id}".format(
            project_id=project_id)
        if detail:
            url = base_url + '/detail'
        else:
            url = base_url
        return self._get(url, response_key="quota", headers=headers)

    def defaults(self, project_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/quotas/{project_id}/defaults".format(
            project_id=project_id)
        return self._get(url, response_key="quota", headers=headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures
import requests
import testtools


class TestCaseShell(testtools.TestCase):
    TEST_REQUEST_BASE = {
        'verify': True,
    }

    def setUp(self):
        super(TestCaseShell, self).setUp()
        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))


class TestResponse(requests.Response):
    """Class used to wrap requests.Response and provide some

       convenience to initialize with a dict.
    """

    def __init__(self, data):
        self._text = None
        super(TestResponse, self)
        if isinstance(data, dict):
            self.status_code = data.get('status_code')
            self.headers = data.get('headers')
            self.reason = data.get('reason', '')
            # Fake the text attribute to streamline Response creation
            self._text = data.get('text')
        else:
            self.status_code = data

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)

    @property
    def text(self):
        return self._text



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 operation_log action implementations"""

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.i18n import _


class ListOperationLogs(command.Lister):
    _description = _("List operation_logs.")

    log = logging.getLogger(__name__ + ".ListOperationLogs")

    def get_parser(self, prog_name):
        parser = super(ListOperationLogs, self).get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=_('Include all projects (admin only)'),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Filter results by status'),
        )
        parser.add_argument(
            '--marker',
            metavar='<operation_log>',
            help=_('The last operation_log ID of the previous page'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-operation_logs>',
            help=_('Maximum number of operation_logs to display'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc), "
                   "multiple keys and directions can be "
                   "specified separated by comma"),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Filter results by a project(admin only)')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection
        all_projects = bool(parsed_args.project) or parsed_args.all_projects

        search_opts = {
            'all_tenants': all_projects,
            'project_id': parsed_args.project,
            'status': parsed_args.status,
        }

        data = data_protection_client.operation_logs.list(
            search_opts=search_opts, marker=parsed_args.marker,
            limit=parsed_args.limit, sort=parsed_args.sort)

        column_headers = ['Id', 'Operation Type', 'Checkpoint id',
                          'Plan Id', 'Provider id', 'Restore Id',
                          'Scheduled Operation Id', 'Status',
                          'Started At', 'Ended At', 'Error Info',
                          'Extra Info']

        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers
                ) for s in data))


class ShowOperationLog(command.ShowOne):
    _description = "Shows operation_log details"

    def get_parser(self, prog_name):
        parser = super(ShowOperationLog, self).get_parser(prog_name)
        parser.add_argument(
            'operation_log',
            metavar="<operation_log>",
            help=_('The UUID of the operation_log.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        operation_log = osc_utils.find_resource(client.operation_logs,
                                                parsed_args.operation_log)

        operation_log._info.pop("links", None)
        return zip(*sorted(operation_log._info.items()))



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
from osc_lib.tests import utils


class TestDataProtection(utils.TestCommand):

    def setUp(self):
        super(TestDataProtection, self).setUp()

        self.app.client_manager.data_protection = mock.Mock()
        self.app.client_manager.network = mock.Mock()
        self.app.client_manager.compute = mock.Mock()
        self.app.client_manager.volume = mock.Mock()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from karborclient.common import base


class Checkpoint(base.Resource):
    def __repr__(self):
        return "<Checkpoint %s>" % self._info

    def get(self):
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return
        plan = self.protection_plan
        if plan is not None:
            provider_id = plan.get("provider_id")
            new = self.manager.get(provider_id, self.id)
            if new:
                self._add_details(new._info)
        else:
            return


class CheckpointManager(base.ManagerWithFind):
    resource_class = Checkpoint

    def create(self, provider_id, plan_id, checkpoint_extra_info=None):
        body = {'checkpoint': {'plan_id': plan_id,
                               'extra-info': checkpoint_extra_info}}
        url = "/providers/{provider_id}/" \
              "checkpoints" .format(provider_id=provider_id)
        return self._create(url, body, 'checkpoint')

    def delete(self, provider_id, checkpoint_id):
        path = '/providers/{provider_id}/checkpoints/' \
               '{checkpoint_id}'.format(provider_id=provider_id,
                                        checkpoint_id=checkpoint_id)
        return self._delete(path)

    def get(self, provider_id, checkpoint_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = '/providers/{provider_id}/checkpoints/' \
            '{checkpoint_id}'.format(provider_id=provider_id,
                                     checkpoint_id=checkpoint_id)
        return self._get(url, response_key="checkpoint", headers=headers)

    def list(self, provider_id=None, search_opts=None, marker=None,
             limit=None, sort_key=None, sort_dir=None, sort=None):
        """Lists all checkpoints.

        :param provider_id:
        :param search_opts: Search options to filter out checkpoints.
        :param marker: Begin returning checkpoints that appear later in the
                       checkpoints list.
        :param limit: Maximum number of checkpoints to return.
        :param sort_key: Key to be sorted; deprecated in kilo
        :param sort_dir: Sort direction, should be 'desc' or 'asc'; deprecated
                         in kilo
        :param sort: Sort information
        :rtype: list of :class:`checkpoint`
        """

        url = self._build_checkpoints_list_url(
            provider_id,
            search_opts=search_opts, marker=marker,
            limit=limit, sort_key=sort_key,
            sort_dir=sort_dir, sort=sort)
        return self._list(url, 'checkpoints')

    def _build_checkpoints_list_url(self, provider_id,
                                    search_opts=None, marker=None, limit=None,
                                    sort_key=None, sort_dir=None, sort=None):

        if search_opts is None:
            search_opts = {}

        query_params = {}
        for key, val in search_opts.items():
            if val:
                query_params[key] = val

        if marker:
            query_params['marker'] = marker

        if limit:
            query_params['limit'] = limit

        if sort:
            query_params['sort'] = self._format_sort_param(sort)
        else:
            # sort_key and sort_dir deprecated in kilo, prefer sort
            if sort_key:
                query_params['sort_key'] = self._format_sort_key_param(
                    sort_key)

            if sort_dir:
                query_params['sort_dir'] = self._format_sort_dir_param(
                    sort_dir)

        # Transform the dict to a sequence of two-element tuples in fixed
        # order, then the encoded string will be consistent in Python 2&3.
        query_string = ""
        if query_params:
            params = sorted(query_params.items(), key=lambda x: x[0])
            query_string = "?%s" % parse.urlencode(params)

        return ("/providers/%(provider_id)s"
                "/checkpoints%(query_string)s" %
                {"provider_id": provider_id,
                 "query_string": query_string})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'protectable_type': {}})
mock_instances_request_return = ({}, {'instances': {}})
mock_instance_request_return = ({}, {'instance': {}})


class ProtectablesTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_protectables(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.protectables.list()
        mock_request.assert_called_with(
            'GET',
            '/protectables', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_get_protectables(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.protectables.get('OS::Cinder::Volume')
        mock_request.assert_called_with(
            'GET',
            '/protectables/OS::Cinder::Volume', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_get_protectables_instance(self, mock_request):
        mock_request.return_value = mock_instance_request_return
        cs.protectables.get_instance('OS::Cinder::Volume', '1')
        mock_request.assert_called_with(
            'GET',
            '/protectables/OS::Cinder::Volume/'
            'instances/1', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_protectables_instances(self, mock_request):
        mock_request.return_value = mock_instances_request_return
        cs.protectables.list_instances('OS::Cinder::Volume')
        mock_request.assert_called_with(
            'GET',
            '/protectables/OS::Cinder::Volume/'
            'instances', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_protectables_instances_with_marker_limit(self, mock_request):
        mock_request.return_value = mock_instances_request_return
        cs.protectables.list_instances('OS::Cinder::Volume',
                                       marker=1234, limit=2)
        mock_request.assert_called_with(
            'GET',
            '/protectables/OS::Cinder::Volume/'
            'instances?limit=2&marker=1234', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_protectables_instances_with_sort_key_dir(self, mock_request):
        mock_request.return_value = mock_instances_request_return
        cs.protectables.list_instances('OS::Cinder::Volume',
                                       sort_key='id', sort_dir='asc')
        mock_request.assert_called_with(
            'GET',
            '/protectables/OS::Cinder::Volume/'
            'instances?sort_dir=asc&sort_key=id', headers={})

    def test_list_protectables_instances_with_invalid_sort_key(self):
        self.assertRaises(ValueError,
                          cs.protectables.list_instances, 'OS::Cinder::Volume',
                          sort_key='invalid', sort_dir='asc')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from karborclient.common.apiclient import exceptions


def extract_resources(args):
    resources = []
    for data in args.resources.split(','):
        if '=' in data and len(data.split('=')) in [3, 4]:
            resource = dict(zip(['id', 'type', 'name', 'extra_info'],
                                data.split('=')))
            if resource.get('extra_info'):
                resource['extra_info'] = jsonutils.loads(
                    resource.get('extra_info'))
        else:
            raise exceptions.CommandError(
                "Unable to parse parameter resources. "
                "The keys of resource are id , type, name and "
                "extra_info. The extra_info field is optional.")
        resources.append(resource)
    return resources


def check_resources(cs, resources):
    # check the resource whether it is available
    for resource in resources:
        try:
            instance = cs.protectables.get_instance(
                resource["type"], resource["id"])
        except exceptions.NotFound:
            raise exceptions.CommandError(
                "The resource: %s can not be found." % resource["id"])
        else:
            if instance is None:
                raise exceptions.CommandError(
                    "The resource: %s is invalid." % resource["id"])


def extract_parameters(args):
    if all((args.parameters, args.parameters_json)):
        raise exceptions.CommandError(
            "Must provide parameters or parameters-json, not both")
    if not any((args.parameters, args.parameters_json)):
        return {}

    if args.parameters_json:
        return jsonutils.loads(args.parameters_json)
    parameters = {}
    for resource_params in args.parameters:
        resource_type = None
        resource_id = None
        parameter = {}
        for param_kv in resource_params.split(','):
            try:
                key, value = param_kv.split('=')
            except Exception:
                raise exceptions.CommandError(
                    'parameters must be in the form: key1=val1,key2=val2,...'
                )
            if key == "resource_type":
                resource_type = value
            elif key == "resource_id":
                if not uuidutils.is_uuid_like(value):
                    raise exceptions.CommandError('resource_id must be a uuid')
                resource_id = value
            else:
                parameter[key] = value
        if resource_type is None:
            raise exceptions.CommandError(
                'Must specify resource_type for parameters'
            )
        if resource_id is None:
            resource_key = resource_type
        else:
            resource_key = "%s#%s" % (resource_type, resource_id)
        parameters[resource_key] = parameter

    return parameters


def extract_instances_parameters(args):
    parameters = {}
    for parameter in args.parameters:
        if '=' in parameter:
            (key, value) = parameter.split('=', 1)
        else:
            key = parameter
            value = None

        parameters[key] = value
    return parameters


def extract_extra_info(args):
    checkpoint_extra_info = {}
    for data in args.extra_info:
        # unset doesn't require a val, so we have the if/else
        if '=' in data:
            (key, value) = data.split('=', 1)
        else:
            key = data
            value = None

        checkpoint_extra_info[key] = value
    return checkpoint_extra_info


def extract_properties(args):
    properties = {}
    if args.properties is None:
        return properties
    for data in args.properties.split(','):
        if '=' in data:
            (resource_key, resource_value) = data.split('=', 1)
        else:
            raise exceptions.CommandError(
                "Unable to parse parameter properties.")

        properties[resource_key] = resource_value
    return properties


def extract_operation_definition(args):
    operation_definition = {}
    for data in args.operation_definition.split(','):
        if '=' in data:
            (resource_key, resource_value) = data.split('=', 1)
        else:
            raise exceptions.CommandError(
                "Unable to parse parameter operation_definition.")

        operation_definition[resource_key] = resource_value
    return operation_definition



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 checkpoint action implementations"""

import json
from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.common.apiclient import exceptions
from karborclient.i18n import _
from karborclient import utils


def format_checkpoint(checkpoint_info):
    if 'protection_plan' in checkpoint_info:
        plan = checkpoint_info['protection_plan']
        checkpoint_info['protection_plan'] = "Name: %s\nId: %s" % (
            plan['name'], plan['id'])
    if 'resource_graph' in checkpoint_info:
        checkpoint_info['resource_graph'] = json.dumps(json.loads(
            checkpoint_info['resource_graph']), indent=2, sort_keys=True)
    checkpoint_info.pop("links", None)


class ListCheckpoints(command.Lister):
    _description = _("List checkpoints.")

    log = logging.getLogger(__name__ + ".ListCheckpoints")

    def get_parser(self, prog_name):
        parser = super(ListCheckpoints, self).get_parser(prog_name)
        parser.add_argument(
            'provider_id',
            metavar='<provider_id>',
            help=_('ID of provider.'),
        )
        parser.add_argument(
            '--plan_id',
            metavar='<plan_id>',
            default=None,
            help=_('Filters results by a plan ID. Default=None.'),
        )
        parser.add_argument(
            '--start_date',
            type=str,
            metavar='<start_date>',
            default=None,
            help=_('Filters results by a start date("Y-m-d"). Default=None.'),
        )
        parser.add_argument(
            '--end_date',
            type=str,
            metavar='<end_date>',
            default=None,
            help=_('Filters results by a end date("Y-m-d"). Default=None.'),
        )
        parser.add_argument(
            '--project_id',
            metavar='<project_id>',
            default=None,
            help=_('Filters results by a project ID. Default=None.'),
        )
        parser.add_argument(
            '--marker',
            metavar='<checkpoint>',
            help=_('The last checkpoint ID of the previous page.'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-checkpoints>',
            help=_('Maximum number of checkpoints to display.'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc), "
                   "multiple keys and directions can be "
                   "specified separated by comma"),
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection

        search_opts = {
            'plan_id': parsed_args.plan_id,
            'start_date': parsed_args.start_date,
            'end_date': parsed_args.end_date,
            'project_id': parsed_args.project_id,
        }

        data = data_protection_client.checkpoints.list(
            provider_id=parsed_args.provider_id, search_opts=search_opts,
            marker=parsed_args.marker, limit=parsed_args.limit,
            sort=parsed_args.sort)

        column_headers = ['Id', 'Project id', 'Status', 'Protection plan',
                          'Metadata', 'Created at']

        def plan_formatter(plan):
            return "Name: %s\nId: %s" % (plan['name'],
                                         plan['id'])
        formatters = {"Protection plan": plan_formatter}
        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers, formatters=formatters
                ) for s in data))


class ShowCheckpoint(command.ShowOne):
    _description = "Shows checkpoint details"

    def get_parser(self, prog_name):
        parser = super(ShowCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'provider_id',
            metavar="<provider_id>",
            help=_('Id of provider.')
        )
        parser.add_argument(
            'checkpoint_id',
            metavar="<checkpoint_id>",
            help=_('Id of checkpoint.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        checkpoint = client.checkpoints.get(parsed_args.provider_id,
                                            parsed_args.checkpoint_id)
        format_checkpoint(checkpoint._info)
        return zip(*sorted(checkpoint._info.items()))


class CreateCheckpoint(command.ShowOne):
    _description = "Creates a checkpoint"

    def get_parser(self, prog_name):
        parser = super(CreateCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'provider_id',
            metavar='<provider_id>',
            help=_('ID of provider.')
        )
        parser.add_argument(
            'plan_id',
            metavar='<plan_id>',
            help=_('ID of plan.')
        )
        parser.add_argument(
            '--extra_info',
            type=str,
            nargs='*',
            metavar='<key=value>',
            default=None,
            help=_('The extra info of a checkpoint.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        checkpoint_extra_info = None
        if parsed_args.extra_info is not None:
            checkpoint_extra_info = utils.extract_extra_info(parsed_args)
        checkpoint = client.checkpoints.create(parsed_args.provider_id,
                                               parsed_args.plan_id,
                                               checkpoint_extra_info)
        format_checkpoint(checkpoint._info)
        return zip(*sorted(checkpoint._info.items()))


class DeleteCheckpoint(command.Command):
    _description = "Delete checkpoint"

    log = logging.getLogger(__name__ + ".DeleteCheckpoint")

    def get_parser(self, prog_name):
        parser = super(DeleteCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'provider_id',
            metavar='<provider_id>',
            help=_('Id of provider.')
        )
        parser.add_argument(
            'checkpoint',
            metavar='<checkpoint>',
            nargs="+",
            help=_('Id of checkpoint.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        failure_count = 0
        for checkpoint_id in parsed_args.checkpoint:
            try:
                client.checkpoints.delete(parsed_args.provider_id,
                                          checkpoint_id)
            except exceptions.NotFound:
                failure_count += 1
                self.log.error(
                    "Failed to delete '{0}'; checkpoint not found".
                    format(checkpoint_id))
        if failure_count == len(parsed_args.checkpoint):
            raise exceptions.CommandError(
                "Unable to find and delete any of the "
                "specified checkpoint.")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import re
import sys

import fixtures
from keystoneauth1 import fixture
from keystoneauth1.fixture import v2 as ks_v2_fixture
import mock
from oslo_log import handlers
from oslo_log import log
import six
from testtools import matchers

from karborclient.common.apiclient import exceptions
import karborclient.shell
from karborclient.tests.unit import base


FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_TENANT_NAME': 'tenant_name',
            'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV2 = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
             'OS_TENANT_ID': 'tenant_id',
             'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV_v3 = {'OS_USERNAME': 'username',
               'OS_PASSWORD': 'password',
               'OS_TENANT_ID': 'tenant_id',
               'OS_USER_DOMAIN_NAME': 'domain_name',
               'OS_AUTH_URL': 'http://no.where/v3'}


def _create_ver_list(versions):
    return {'versions': {'values': versions}}


class TestArgs(object):
    package_version = ''
    karbor_repo_url = 'http://127.0.0.1'
    exists_action = ''
    is_public = False
    categories = []


class ShellTest(base.TestCaseShell):

    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))


class ShellCommandTest(ShellTest):

    _msg_no_tenant_project = ('You must provide a project name or project'
                              ' id via --os-project-name, --os-project-id,'
                              ' env[OS_PROJECT_ID] or env[OS_PROJECT_NAME].'
                              ' You may use os-project and os-tenant'
                              ' interchangeably.',)

    def setUp(self):
        super(ShellCommandTest, self).setUp()

        def get_auth_endpoint(bound_self, args):
            return ('test', {})
        self.useFixture(fixtures.MonkeyPatch(
            'karborclient.shell.KarborShell._get_endpoint_and_kwargs',
            get_auth_endpoint))
        self.client = mock.MagicMock()

        # To prevent log descriptors from being closed during
        # shell tests set a custom StreamHandler
        self.logger = log.getLogger(None).logger
        self.logger.level = logging.DEBUG
        self.color_handler = handlers.ColorHandler(sys.stdout)
        self.logger.addHandler(self.color_handler)

    def tearDown(self):
        super(ShellTest, self).tearDown()
        self.logger.removeHandler(self.color_handler)

    def shell(self, argstr, exitcodes=(0,)):
        orig = sys.stdout
        orig_stderr = sys.stderr
        try:
            sys.stdout = six.StringIO()
            sys.stderr = six.StringIO()
            _shell = karborclient.shell.KarborShell()
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertIn(exc_value.code, exitcodes)
        finally:
            stdout = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
            stderr = sys.stderr.getvalue()
            sys.stderr.close()
            sys.stderr = orig_stderr
        return (stdout, stderr)

    def register_keystone_discovery_fixture(self, mreq):
        v2_url = "http://no.where/v2.0"
        v2_version = fixture.V2Discovery(v2_url)
        mreq.register_uri('GET', v2_url, json=_create_ver_list([v2_version]),
                          status_code=200)

    def register_keystone_token_fixture(self, mreq):
        v2_token = ks_v2_fixture.Token(token_id='token')
        service = v2_token.add_service('application-catalog')
        service.add_endpoint('http://no.where', region='RegionOne')
        mreq.register_uri('POST',
                          'http://no.where/v2.0/tokens',
                          json=v2_token,
                          status_code=200)

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')

    def test_help(self):
        required = [
            '.*?^usage: karbor',
            '.*?^\s+plan-create\s+Creates a plan.',
            '.*?^See "karbor help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('help')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_on_subcommand(self):
        required = [
            '.*?^usage: karbor plan-create',
            '.*?^Creates a plan.',
        ]
        stdout, stderr = self.shell('help plan-create')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_no_options(self):
        required = [
            '.*?^usage: karbor',
            '.*?^\s+plan-create\s+Creates a plan',
            '.*?^See "karbor help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_no_username(self):
        required = ('You must provide a username via either --os-username or '
                    'env[OS_USERNAME] or a token via --os-auth-token or '
                    'env[OS_AUTH_TOKEN]',)
        self.make_env(exclude='OS_USERNAME')
        try:
            self.shell('plan-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_name(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_TENANT_NAME')
        try:
            self.shell('plan-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_id(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_TENANT_ID', fake_env=FAKE_ENV2)
        try:
            self.shell('plan-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_auth_url(self):
        required = ('You must provide an auth url'
                    ' via either --os-auth-url or via env[OS_AUTH_URL]',)
        self.make_env(exclude='OS_AUTH_URL')
        try:
            self.shell('plan-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import triggers as osc_triggers
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import triggers


TRIGGER_INFO = {
    "id": "2a9ce1f3-cc1a-4516-9435-0ebb13caa398",
    "name": "My backup trigger",
    "type": "time",
    "properties": {
        "format": "calendar",
        "pattern": "BEGIN:VEVENT\\nRRULE:FREQ=HOURLY;INTERVAL=1;\\nEND:VEVENT",
        "start_time": "2015-12-17T08:30:00",
        "end_time": "2016-03-17T08:30:00",
        "window": "3600"
    }
}


class TestTriggers(fakes.TestDataProtection):
    def setUp(self):
        super(TestTriggers, self).setUp()
        self.triggers_mock = self.app.client_manager.data_protection.triggers
        self.triggers_mock.reset_mock()


class TestListTriggers(TestTriggers):
    def setUp(self):
        super(TestListTriggers, self).setUp()
        self.triggers_mock.list.return_value = [triggers.Trigger(
            None, copy.deepcopy(TRIGGER_INFO))]

        # Command to test
        self.cmd = osc_triggers.ListTriggers(self.app, None)

    def test_triggers_list(self):
        arglist = ['--name', 'My backup trigger']
        verifylist = [('name', 'My backup trigger')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Name', 'Type', 'Properties'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [("2a9ce1f3-cc1a-4516-9435-0ebb13caa398",
                          "My backup trigger",
                          "time",
                          {"format": "calendar",
                           "pattern": "BEGIN:VEVENT\\nRRULE:FREQ=HOURLY;INTERVAL=1;\\nEND:VEVENT",  # noqa
                           "start_time": "2015-12-17T08:30:00",
                           "end_time": "2016-03-17T08:30:00",
                           "window": "3600"})]
        self.assertEqual(expected_data, list(data))


class TestCreateTrigger(TestTriggers):
    def setUp(self):
        super(TestCreateTrigger, self).setUp()
        self.triggers_mock.create.return_value = triggers.Trigger(
            None, copy.deepcopy(TRIGGER_INFO))
        # Command to test
        self.cmd = osc_triggers.CreateTrigger(self.app, None)

    def test_trigger_create(self):
        arglist = ['My backup trigger',
                   'time',
                   "'format'='calendar',"
                   "'pattern'='BEGIN:VEVENT\\nRRULE:FREQ=HOURLY;INTERVAL=1;\\nEND:VEVENT',"  # noqa
                   "'start_time'='2015-12-17T08:30:00',"
                   "'end_time'='2016-03-17T08:30:00',"
                   "'window'='3600'"]
        verifylist = [('name', 'My backup trigger'),
                      ('type', 'time'),
                      ('properties', "'format'='calendar',"
                                     "'pattern'='BEGIN:VEVENT\\nRRULE:FREQ=HOURLY;INTERVAL=1;\\nEND:VEVENT',"  # noqa
                                     "'start_time'='2015-12-17T08:30:00',"
                                     "'end_time'='2016-03-17T08:30:00',"
                                     "'window'='3600'")]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.triggers_mock.create.assert_called_once_with(
            'My backup trigger',
            'time',
            "'format'='calendar',"
            "'pattern'='BEGIN:VEVENT\\nRRULE:FREQ=HOURLY;INTERVAL=1;\\nEND:VEVENT',"  # noqa
            "'start_time'='2015-12-17T08:30:00',"
            "'end_time'='2016-03-17T08:30:00',"
            "'window'='3600'")


class TestUpdateTrigger(TestTriggers):
    def setUp(self):
        super(TestUpdateTrigger, self).setUp()
        self.triggers_mock.get.return_value = triggers.Trigger(
            None, copy.deepcopy(TRIGGER_INFO))
        self.triggers_mock.update.return_value = triggers.Trigger(
            None, copy.deepcopy(TRIGGER_INFO))
        # Command to test
        self.cmd = osc_triggers.UpdateTrigger(self.app, None)

    def test_trigger_update(self):
        arglist = ['2a9ce1f3-cc1a-4516-9435-0ebb13caa398',
                   '--name', 'My backup trigger']
        verifylist = [('trigger_id', '2a9ce1f3-cc1a-4516-9435-0ebb13caa398'),
                      ('name', 'My backup trigger')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.triggers_mock.update.assert_called_once_with(
            '2a9ce1f3-cc1a-4516-9435-0ebb13caa398',
            {'name': 'My backup trigger'})


class TestDeleteTrigger(TestTriggers):
    def setUp(self):
        super(TestDeleteTrigger, self).setUp()
        self.triggers_mock.get.return_value = triggers.Trigger(
            None, copy.deepcopy(TRIGGER_INFO))
        # Command to test
        self.cmd = osc_triggers.DeleteTrigger(self.app, None)

    def test_trigger_delete(self):
        arglist = ['2a9ce1f3-cc1a-4516-9435-0ebb13caa398']
        verifylist = [('trigger', ['2a9ce1f3-cc1a-4516-9435-0ebb13caa398'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.triggers_mock.delete.assert_called_once_with(
            '2a9ce1f3-cc1a-4516-9435-0ebb13caa398')


class TestShowTrigger(TestTriggers):
    def setUp(self):
        super(TestShowTrigger, self).setUp()
        self._trigger_info = copy.deepcopy(TRIGGER_INFO)
        self.triggers_mock.get.return_value = triggers.Trigger(
            None, self._trigger_info)

        # Command to test
        self.cmd = osc_triggers.ShowTrigger(self.app, None)

    def test_trigger_show(self):
        arglist = ['2a9ce1f3-cc1a-4516-9435-0ebb13caa398']
        verifylist = [('trigger', '2a9ce1f3-cc1a-4516-9435-0ebb13caa398')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            'id', 'name', 'properties', 'type')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        self.assertEqual(self._trigger_info['id'], data[0])
        self.assertEqual(self._trigger_info['name'], data[1])
        self.assertEqual(self._trigger_info['properties'], data[2])
        self.assertEqual(self._trigger_info['type'], data[3])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 quotas action implementations"""

from osc_lib.command import command


def quota_set_pretty_show(quotas):
    """Convert quotas object to dict and display."""

    new_quotas = []
    for quota_k, quota_v in sorted(quotas.to_dict().items()):
        if isinstance(quota_v, dict):
            quota_v = '\n'.join(
                ['%s = %s' % (k, v) for k, v in sorted(quota_v.items())])
        new_quotas.append((quota_k, quota_v))

    return new_quotas


class ShowQuotas(command.ShowOne):
    _description = "Shows Quotas"

    def get_parser(self, prog_name):
        parser = super(ShowQuotas, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant',
            metavar='<tenant>',
            default=None,
            help='ID of tenant to list the quotas for.')
        parser.add_argument(
            '--detail',
            action='store_true',
            help='Optional flag to indicate whether to show quota in detail. '
                 'Default false.')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        project_id = parsed_args.tenant or client.http_client.get_project_id()
        kwargs = {
            "project_id": project_id,
            "detail": parsed_args.detail,
        }
        result = client.quotas.get(**kwargs)
        quotas = quota_set_pretty_show(result)
        return zip(*sorted(quotas))


class ShowDefaultQuotas(command.ShowOne):
    _description = "Shows default Quotas"

    def get_parser(self, prog_name):
        parser = super(ShowDefaultQuotas, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant',
            metavar='<tenant>',
            default=None,
            help='ID of tenant to list the quotas for.')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        project_id = parsed_args.tenant or client.http_client.get_project_id()

        result = client.quotas.defaults(project_id)
        quotas = quota_set_pretty_show(result)

        return zip(*sorted(quotas))


class UpdateQuotas(command.ShowOne):
    _description = "Updates Quotas"

    def get_parser(self, prog_name):
        parser = super(UpdateQuotas, self).get_parser(prog_name)
        parser.add_argument(
            'tenant',
            metavar='<tenant>',
            help='ID of tenant to set the quotas for.')
        parser.add_argument(
            '--plans',
            metavar='<plans>',
            type=int,
            default=None,
            help='New value for the "plans" quota.')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        project_id = parsed_args.tenant
        data = {
            "plans": parsed_args.plans,
        }
        result = client.quotas.update(project_id, data)
        quotas = quota_set_pretty_show(result)

        return zip(*sorted(quotas))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 verification action implementations"""

import functools
import json

from oslo_utils import uuidutils

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.common.apiclient import exceptions
from karborclient.i18n import _
from karborclient import utils


def format_verification(verification_info):
    for key in ('parameters', 'resources_status',
                'resources_reason'):
        if key not in verification_info:
            continue
        verification_info[key] = json.dumps(verification_info[key],
                                            indent=2, sort_keys=True)
    verification_info.pop("links", None)


class ListVerifications(command.Lister):
    _description = _("List verifications.")

    log = logging.getLogger(__name__ + ".ListVerifications")

    def get_parser(self, prog_name):
        parser = super(ListVerifications, self).get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=_('Include all projects (admin only)'),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Filter results by status'),
        )
        parser.add_argument(
            '--marker',
            metavar='<verification>',
            help=_('The last verification ID of the previous page'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-verifications>',
            help=_('Maximum number of verifications to display'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc), "
                   "multiple keys and directions can be "
                   "specified separated by comma"),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Filter results by a project(admin only)')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection
        all_projects = bool(parsed_args.project) or parsed_args.all_projects

        search_opts = {
            'all_tenants': all_projects,
            'project_id': parsed_args.project,
            'status': parsed_args.status,
        }

        data = data_protection_client.verifications.list(
            search_opts=search_opts, marker=parsed_args.marker,
            limit=parsed_args.limit, sort=parsed_args.sort)

        column_headers = ['Id', 'Project id', 'Provider id', 'Checkpoint id',
                          'Parameters', 'Status']

        json_dumps = functools.partial(json.dumps, indent=2, sort_keys=True)
        formatters = {
            "Parameters": json_dumps,
        }
        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers, formatters=formatters,
                ) for s in data))


class ShowVerification(command.ShowOne):
    _description = "Shows verification details"

    def get_parser(self, prog_name):
        parser = super(ShowVerification, self).get_parser(prog_name)
        parser.add_argument(
            'verification',
            metavar="<verification>",
            help=_('The UUID of the verification.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        verification = osc_utils.find_resource(client.verifications,
                                               parsed_args.verification)

        format_verification(verification._info)
        return zip(*sorted(verification._info.items()))


class CreateVerification(command.ShowOne):
    _description = "Creates a verification"

    def get_parser(self, prog_name):
        parser = super(CreateVerification, self).get_parser(prog_name)
        parser.add_argument(
            'provider_id',
            metavar='<provider_id>',
            help=_('The UUID of the provider.')
        )
        parser.add_argument(
            'checkpoint_id',
            metavar='<checkpoint_id>',
            help=_('The UUID of the checkpoint.')
        )
        parser.add_argument(
            '--parameters-json',
            type=str,
            dest='parameters_json',
            metavar='<parameters>',
            default=None,
            help=_('Verification parameters in json format.')
        )
        parser.add_argument(
            '--parameters',
            action='append',
            metavar='resource_type=<type>[,resource_id=<id>,key=val,...]',
            default=[],
            help=_("Verification parameters, may be specified multiple times. "
                   "resource_type: type of resource to apply parameters. "
                   "resource_id: limit the parameters to a specific resource. "
                   "Other keys and values: according to provider\'s "
                   "verification schema.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        if not uuidutils.is_uuid_like(parsed_args.provider_id):
            raise exceptions.CommandError(
                "Invalid provider id provided.")
        if not uuidutils.is_uuid_like(parsed_args.checkpoint_id):
            raise exceptions.CommandError(
                "Invalid checkpoint id provided.")

        verification_parameters = utils.extract_parameters(parsed_args)
        verification = client.verifications.create(parsed_args.provider_id,
                                                   parsed_args.checkpoint_id,
                                                   verification_parameters)
        format_verification(verification._info)
        return zip(*sorted(verification._info.items()))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib import utils

LOG = logging.getLogger(__name__)

DEFAULT_DATA_PROTECTION_API_VERSION = '1'
API_VERSION_OPTION = 'os_data_protection_api_version'
API_NAME = 'data_protection'
API_VERSIONS = {
    '1': 'karborclient.v1.client.Client',
}


def make_client(instance):
    """Returns a data protection service client"""
    data_protection_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)
    LOG.debug('Instantiating data protection client: %s',
              data_protection_client)
    client = data_protection_client(
        auth=instance.auth,
        session=instance.session,
        service_type="data-protect"
    )

    return client


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-data-protection-api-version',
        metavar='<data-protection-api-version>',
        default=utils.env(
            'OS_DATA_PROTECTION_API_VERSION',
            default=DEFAULT_DATA_PROTECTION_API_VERSION),
        help='Data protection API version, default=' +
             DEFAULT_DATA_PROTECTION_API_VERSION +
             ' (Env: OS_DATA_PROTECTION_API_VERSION)')
    return parser
